from importlib import import_module
import os
import glob, imp


def _load(path):
    name, ext = os.path.splitext(os.path.basename(path))
    return name, imp.load_source(name, path)

def get_providers():
    path = os.path.relpath(os.path.dirname(__file__))

    files = glob.glob(os.path.join(path,'[!_]*.py'))
    modules = dict(_load(path) for path in files)
    providers = []
    for name, module in modules.items():
        func = None
        try:
            func = getattr(module, 'lookup')
        except AttributeError:
            continue
        providers.append((name, func))

    return providers


from services import get_providers

providers = get_providers()

def embed(url, **kwargs):
    res = None
    for service, lookup in providers:
        res = lookup(url, **kwargs)
        if res:
            break
    return res


from oembed import Consumer, NoEndpointError


def lookup(url, **kwargs):
    consumer = Consumer(
        [
            ('http://vimeo.com/*', 'http://www.vimeo.com/api/oembed.json'),
            ('http://vimeo.com/*', 'http://www.vimeo.com/api/oembed.json'),
            ('http://www.vimeo.com/groups/*/*', 'http://www.vimeo.com/api/oembed.json'),
            ('http://www.vimeo.com/groups/*/*', 'http://www.vimeo.com/api/oembed.json'),
        ]
    )
    try:
        res = consumer.lookup(url, **kwargs)
    except NoEndpointError:
        res = None
    return res


from oembed import Consumer, NoEndpointError, NotFoundError
import re


def lookup(url, **kwargs):
    consumer = Consumer(
        [
            ('http://*.flickr.com/*', 'http://www.flickr.com/services/oembed/'),
        ]
    )
    try:
        res = consumer.lookup(url, **kwargs)
    except NoEndpointError, NotFoundError:
        res = None

    # since our friends at flickr do not return the thumbnail_url,
    # we will hack it ourselves, thumbnails are always in .jpg format
    # http://www.flickr.com/services/api/misc.urls.html
    if res:
        sre = re.search('_[mstzb].(?P<ext>jpg|gif|png)', res['thumbnail_url'])
        if sre:
            thumb_url = '%s_t.jpg' % res['thumbnail_url'][:sre.start()]
            ext = sre.group('ext')
            # the image can be small, medium or whatever, we want original
            url = '%s.%s' % (res['thumbnail_url'][:sre.start()], sre.group('ext'))
            res['url'] = url
        else:
            thumb_url = '%s_t.jpg' % res['url'][:-4]

        res[u'thumbnail_url'] = thumb_url

    return res



""" Quickmeme processor --http://www.quickmeme.com/--

    Quickmeme doesnt provide oEmbed support
    Let's rip it apart

    Normal link: http://www.quickmeme.com/meme/352890/
    With some vars: http://www.quickmeme.com/meme/7fzt/#by=ad
    Image: http://i.qkme.me/7fzt.jpg
    Thumb: http://t.qkme.me/7fzt.jpg
"""
from urlparse import urlparse
import re

def lookup(url, **kwargs):
    normal = re.search('quickmeme.com', url)
    direct = re.search('qkme.me', url)
    if not normal and not direct:
        return None
    pr = urlparse(url)
    path = pr.path
    if normal:
        pic_id = re.search('\/\w+\/$', path).group()[1:-1]
    elif direct:
        pic_id = re.search('(?P<pic_id>\w+)(.jpg)?$', path).group('pic_id')
    else:
        raise ValueError("Can't parse your URL %s" % url)

    normal_url = 'http://www.quickmeme.com/meme/%s/' % pic_id
    direct_url = 'http://i.qkme.me/%s.jpg' % pic_id
    thumb_url = 'http://t.qkme.me/%s.jpg' % pic_id

    res = {
        'type': 'photo',
        'version': '1.0',
        'provider_name': 'quickmeme.com',
        'provider_url': 'http://www.quickmeme.com',
        'thumbnail_url': thumb_url,
        'quickmeme_page': normal_url,
        'url': direct_url,
        'width': None,
        'height': None,
        }

    return res


""" Imgur data processor

    Since Imgur doesn't provide oEmbed interface, we have to query data using
    their API, so this is a Wrapper for their API service.

    http://imgur.com/gallery/NiK0X
    http://imgur.com/BynR8
    http://i.imgur.com/BynR8.png
    http://i.imgur.com/BynR8s.png

    images can be png, jpg or gif

    json returned by imgur

    {
        "image":
            {
                "image": {
                            "title": null,
                            "caption": null,
                            "hash": "BynR8",
                            "datetime": "2011-08-30 23:45:13",
                            "type": "image\/jpeg",
                            "animated": "false",
                            "width": 500,
                            "height": 578,
                            "size": 75375,
                            "views": 358449,
                            "bandwidth": 27018093375
                            },
                "links": {
                            "original": "http:\/\/i.imgur.com\/BynR8.jpg",
                            "imgur_page": "http:\/\/imgur.com\/BynR8",
                            "small_square": "http:\/\/i.imgur.com\/BynR8s.jpg",
                            "large_thumbnail": "http:\/\/i.imgur.com\/BynR8l.jpg"
                         }
            }
    }
    
"""
from urlparse import urlparse
import re
import requests


imgur_endpoint = 'http://api.imgur.com/2/image/%s.json'

def lookup(url, **kwargs):
    pr = urlparse(url)
    path = pr.path
    # we are planning to support albums in a near future, not for now or not :P
    if not re.search('(i\.)?imgur.com', url) or re.search('imgur.com\/a\/', url):
        return None
    pattern = r"^https?:\/\/(www\.|i\.)?imgur\.com\/(gallery\/)?(?P<pid>\w{5,})(?P<ptype>\.gif|\.jpg|\.jpeg|\.png)?(\?full)?$"
    res = re.match(pattern, url)
    pid = res.group('pid') if res else None
    if not pid:
        return None
    if len(pid) > 5:
        pid = re.sub(r"(h|m|l|s)?$", '', pid)

    res = requests.get("http://imgur.com/%s?tags" % pid, headers={"User-Agent": "Python Thornbed"})
    # now get the real pid and ptype from that page
    md = re.search('id\=\"nondirect\"[ ]+value\=\"http:\/\/imgur.com\/(?P<pid>\w{5,})', res.content)
    pid = md.group('pid') if md else None
    if not pid:
        return None
    md = re.search('id\=\"direct\"[ ]+value\=\"http://i.imgur.com/\w+(?P<ptype>\.jpg|\.jpeg|\.png|\.gif)', res.content)
    ptype = md.group('ptype') if md else '.jpg'

    # imgur pic_id is ambiguous since the chars for thumb etc... can be part of the id
    # so we check the embed page
    # first get the pseudo id

    # md = pattern.match(url)
    # pid = md[:pid]
    # # TODO: improve this code
    # if pid.length > 5
    # pid = pid.gsub(/(h|m|l|s)?$/, '')
    # end
    # raise Thornbed::NotValid, url if not pid
    #
    # res = HTTParty.get("http://imgur.com/#{pid}?tags", headers: {"User-Agent" => USER_AGENT})
    #
    # # now get the real pid and ptype from that page
    # md = /id\=\"nondirect\"[ ]+value\=\"http:\/\/imgur.com\/(?<pid>\w+)/.match(res.body)
    # pid = md[:pid]
    #
    # raise Thornbed::NotValid, url if not pid
    #
    # md = /id\=\"direct\"[ ]+value\=\"http:\/\/i.imgur.com\/\w+(?<ptype>\.jpg|\.jpeg|\.png|\.gif)/.match(res.body)
    # ptype = md[:ptype] || ".jpg"

    # thumbs are always .jpg
    thumbnail_url = 'http://i.imgur.com/%ss.jpg' % pid
    imgur_page = 'http://imgur.com/%s' % pid
    url = 'http://i.imgur.com/%s%s' % (pid, ptype)
    res = {
        'type': 'photo',
        'version': '1.0',
        'provider_name': 'imgur.com',
        'provider_url': 'http://imgur.com',
        'thumbnail_url': thumbnail_url,
        'imgur_page': imgur_page,
        'url': url,
        'width': '',
        'height': '',
        }

    return res



from setuptools import setup, find_packages

version = '0.4.7'

setup(name='thornbed',
      version=version,
      description="Simple oEmbed wrapper for different services",
      long_description="""\
""",
      classifiers=[
          "Intended Audience :: Developers",
          "License :: OSI Approved :: BSD License",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
          "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
      ], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='oembed youtube vimeo imgur flickr quickmeme',
      author='Esteban Feldman',
      author_email='esteban.feldman@gmail.com',
      url='https://bitbucket.org/eka/thornbed',
      license='BSD',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
          'oEmbed',
          'simplejson',
          'requests',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from urlparse import urlparse
from oembed import Consumer, NoEndpointError
import re


IFRAME_EMBED_CODE = """<iframe class="youtube-player" type="text/html" width="%(width)s" height="%(height)s" src="http://www.youtube.com/embed/%(video_id)s" frameborder="0">
</iframe>"""

def lookup(url, **kwargs):
    consumer = Consumer(
        [
            ('http://youtube.com/watch*', 'http://www.youtube.com/oembed'),
            ('http://youtu.be/*', 'http://www.youtube.com/oembed'),
        ]
    )
    try:
        res = consumer.lookup(url, **kwargs)
    except NoEndpointError:
        return None

    # hack the <iframe> version for html embed code
    # let there be dragons
    video_id = None
    pr = urlparse(url)
    if 'youtube.com' in url:
        query = pr.query
        sre = re.search('v=(?P<id>[A-Za-z0-9_-]*)', query)
        if sre:
            try:
                video_id = sre.group('id')
            except IndexError:
                pass
    elif 'youtu.be' in url:
        path = pr.path
        sre = re.search('[A-Za-z0-9_-]*$', path)
        if sre:
            video_id = sre.group()
    else:
        pass
    if video_id:
        res['html_iframe'] = IFRAME_EMBED_CODE % ({'width': res['width'],'height': res['height'], 'video_id': video_id})
    return res